/*-
 * Copyright (c) 2001
 * Tatsuya Kudoh(CDR/TK),ROYALPANDA.    All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef _KSTEXT_H_
#define _KSTEXT_H_

#include"ksdraw.h"

typedef struct KSTextArea{
	KSWindow win;
	int width;
	int height;

	int cur_line;
	int cx,cy;
	int column;

	int max_size;

	int real_size;
	int real_max_size;

	int (*callback)(struct KSTextArea *area,KSMsg *msg);
	void *call_data;

	int display;

	unsigned *pos1,*pos2;
	unsigned *mark;
	unsigned buf[0];
}KSTextArea;



void KSTextAreaRedraw( KSTextArea *p );
int KSTextAreaForward( KSTextArea *p );
int KSTextAreaBackward( KSTextArea *p );
int KSTextAreaNextLine( KSTextArea *p );
int KSTextAreaPrevLine( KSTextArea *p );
int KSTextAreaPageUp( KSTextArea *p );
int KSTextAreaPageDown( KSTextArea *p );
int KSTextAreaLineHead( KSTextArea *p );
int KSTextAreaLineEnd( KSTextArea *p );
int KSTextAreaInsert( KSTextArea *p, unsigned *str );
int KSTextAreaDeleteBackward( KSTextArea *p, int length );
void KSTextAreaDisplay(KSTextArea *p, int disp );
void KSTextAreaSetMark( KSTextArea *p );
void KSTextAreaResetMark( KSTextArea *p );
KSTextArea *KSCreateTextArea( int x, int y, int width, int height, int maxsize, int (*callback)(KSTextArea *area, KSMsg *msg), void *call_data);


#endif
